########################################################################## 
# Copyright (C) 2022-2023 Kostiantyn Klochko <kklochko@protonmail.com>   #
#                                                                        #
# This file is part of Anitube Simple Notification.                      #
#                                                                        #
# Anitube Simple Notification is free software: you can redistribute     #
# it and/or modify it under the terms of the GNU General Public          #
# License as published by the Free Software Foundation, either version   #
# 3 of the License, or (at your option) any later version.               #
#                                                                        #
# Anitube Simple Notification is distributed in the hope that it will    #
# be useful, but WITHOUT ANY WARRANTY; without even the implied          #
# warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See   #
# the GNU General Public License for more details.                       #
#                                                                        #
# You should have received a copy of the GNU General Public License      #
# along with Anitube Simple Notification. If not, see                    #
# <https://www.gnu.org/licenses/>.                                       #
##########################################################################

"""
This module has all for simplify work with scraping.
"""

import requests
from bs4 import BeautifulSoup
from rich.console import Console
from rich.progress import track
import os

class Scraper:
    """The handler of web connection."""
    def __init__(self, HEADERS, POSTER_PATH="posters"):
        """Initialising the connection information."""
        self.HEADERS, self.POSTER_PATH = HEADERS, POSTER_PATH
        self.mkdir(self.POSTER_PATH)

    def mkdir(self, path):
        if not os.path.isdir(path):
            os.mkdir(path)

    def file_exist(self, path):
        if os.path.isfile(path):
            return True
        return False

    def get_anime(self, url, GETPOSTER = False):
        """
        Return None if response is not 200.
        Otherwise, return [url, title, status].
        """
        response = requests.get(url, headers=self.HEADERS)
        if response.status_code != 200:
            return None
        soup = BeautifulSoup(response.content, 'html.parser')
        data = soup.find('div', class_='rcol', style = 'width:701px; padding:0 0 0 6px;')
        # Getting Title
        title = data.find('h2').get_text(strip=True)
        # Getting Status
        str_find = "\nСерій: "
        str_current = data.get_text()
        str_current = str_current[str_current.find(str_find)+len(str_find):]
        status = str_current[:str_current.find('\n')]
        # Poster
        poster_url = "https://anitube.in.ua" + soup.find('span', class_="story_post").find('img').get('src')
        poster_path = f"{self.POSTER_PATH}/{poster_url.split('/')[-1]}"
        if GETPOSTER and not self.file_exist(poster_path):
            console = Console()
            with console.status("[yellow]Downloading...[/]"):
                img = requests.get(poster_url)
                with open(poster_path,'wb') as file:
                    file.write(img.content)
                console.print(f"[green][DONWLOADED][/] The poster for \"{title}\"")
        return [url, title, status, poster_path]



########################################################################## 
# Copyright (C) 2022-2023 Kostiantyn Klochko <kklochko@protonmail.com>   #
#                                                                        #
# This file is part of Anitube Simple Notification.                      #
#                                                                        #
# Anitube Simple Notification is free software: you can redistribute     #
# it and/or modify it under the terms of the GNU General Public          #
# License as published by the Free Software Foundation, either version   #
# 3 of the License, or (at your option) any later version.               #
#                                                                        #
# Anitube Simple Notification is distributed in the hope that it will    #
# be useful, but WITHOUT ANY WARRANTY; without even the implied          #
# warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See   #
# the GNU General Public License for more details.                       #
#                                                                        #
# You should have received a copy of the GNU General Public License      #
# along with Anitube Simple Notification. If not, see                    #
# <https://www.gnu.org/licenses/>.                                       #
##########################################################################
from .db import DataBase
from .scraper import Scraper
from .notify import Notification
from .config import Config
from rich.console import Console
from rich.progress import track
import time
import os
import platformdirs

def main():
    # Const sections.

    # Console initialising
    console = Console()

    APPNAME = "anitube-simple-notification"
    APPAUTHOR = "KKlochko"
    CONFIG_NAME = "config.toml"
    BASE_DIR = os.path.dirname(__file__)
    DATA_DIR = platformdirs.user_data_dir(APPNAME, APPAUTHOR)

    DB_PATH = os.path.join(DATA_DIR, 'user.db')
    POSTERS_PATH = os.path.join(DATA_DIR, 'posters')

    CONFIG_PATH = platformdirs.user_config_path(APPNAME, APPAUTHOR, CONFIG_NAME)

    # check for existing of dirs
    if not os.path.exists(DATA_DIR):
        os.makedirs(DATA_DIR)

    if not os.path.exists(CONFIG_PATH):
        os.makedirs(CONFIG_PATH)

    config = Config.get_config(CONFIG_PATH, console)

    # Here you can change testing headers to yours.
    HEADERS = {
        'User-Agent': 'Mozilla/5.0 (Windows NT 6.1; Win64; x64; rv:47.0) Gecko/20100101 Firefox/47.0'
    }

    # Default if empty.
    MESSAGE = "New episode."
    WAITING_PERIOD = config.pop('WAITING_PERIOD', 60) # seconds
    POSTERS = config.pop('POSTERS', True) # for use poster as icons of notifications
    urls = config.pop('URLS', [])

    # Initialising objects
    scr = Scraper(HEADERS, POSTERS_PATH)
    db = DataBase(DB_PATH)

    # Checks for new urls in file and add as current state
    # If one of page has updated then notifing.
    # Repeating the checking with the waiting period.
    while True:
        console.print(f"[yellow][DOING][/] Checking for animes [0/{len(urls)}]")
        count = 0
        for url in urls:
            data = scr.get_anime(url, POSTERS)
            if data == None:
                console.print(f"[red][ERROR][/] A conections trouble is occured.")
                continue
            url, title, status, poster_path = data
            console.print(f"[yellow][DOING][/] Checking for \"{title}\" [{count}/{len(urls)}]")
            r = db.add_anime_if(url, title, status, poster_path)
            if r == -1:
                n = Notification(title, MESSAGE, poster_path)
                n.send()
                console.print(f"[blue bold][NOTIFICATION][/] \"{title}\"")
            count+=1
            console.print(f"[green][DONE][/] Checking for \"{title}\" [{count}/{len(urls)}]")
        console.print(f"[yellow][WAITING][/] The next check is after {WAITING_PERIOD} seconds")
        # Sleep while waiting
        for n in track(range(WAITING_PERIOD), description="Waiting..."):
            time.sleep(1)

if __name__ == "__main__":
    main()



########################################################################## 
# Copyright (C) 2022-2023 Kostiantyn Klochko <kklochko@protonmail.com>   #
#                                                                        #
# This file is part of Anitube Simple Notification.                      #
#                                                                        #
# Anitube Simple Notification is free software: you can redistribute     #
# it and/or modify it under the terms of the GNU General Public          #
# License as published by the Free Software Foundation, either version   #
# 3 of the License, or (at your option) any later version.               #
#                                                                        #
# Anitube Simple Notification is distributed in the hope that it will    #
# be useful, but WITHOUT ANY WARRANTY; without even the implied          #
# warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See   #
# the GNU General Public License for more details.                       #
#                                                                        #
# You should have received a copy of the GNU General Public License      #
# along with Anitube Simple Notification. If not, see                    #
# <https://www.gnu.org/licenses/>.                                       #
##########################################################################

"""
This module has all for simplify work with notifications.
"""

from notifypy import Notify

class Notification:
    """The handler of notification."""
    def __init__(self, title, message, icon_path=""):
        """Initialising the notification information."""
        self.title, self.message, self.icon_path = title, message, icon_path

    def send(self):
        """Send the notification."""
        notification = Notify()
        notification.title = self.title
        notification.message = self.message
        notification.icon = self.icon_path
        notification.send()



########################################################################## 
# Copyright (C) 2022-2023 Kostiantyn Klochko <kklochko@protonmail.com>   #
#                                                                        #
# This file is part of Anitube Simple Notification.                      #
#                                                                        #
# Anitube Simple Notification is free software: you can redistribute     #
# it and/or modify it under the terms of the GNU General Public          #
# License as published by the Free Software Foundation, either version   #
# 3 of the License, or (at your option) any later version.               #
#                                                                        #
# Anitube Simple Notification is distributed in the hope that it will    #
# be useful, but WITHOUT ANY WARRANTY; without even the implied          #
# warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See   #
# the GNU General Public License for more details.                       #
#                                                                        #
# You should have received a copy of the GNU General Public License      #
# along with Anitube Simple Notification. If not, see                    #
# <https://www.gnu.org/licenses/>.                                       #
##########################################################################

"""
This module has all for simplify work with sqlite3.
"""

import sqlite3

class DataBase:
    """Handler of connection."""
    def __init__(self, PATH: str = "user.db", TABLE: str = "animes"):
        """Initialising the connection information."""
        self.PATH, self.TABLE = PATH, TABLE
        self.create_table(self.TABLE)

    def create_table(self, table):
        """Create a table in the DB."""
        connect = sqlite3.connect(self.PATH)
        cursor = connect.cursor()
        cursor.execute(f'CREATE TABLE IF NOT EXISTS {table}(id INTEGER PRIMARY KEY, Url TEXT, Title TEXT, Status TEXT, PosterPath TEXT);')
        connect.commit()
        cursor.close()
        connect.close()

    def add_anime(self, url, title, status, poster_path=""):
        """Add entry of a content."""
        connect = sqlite3.connect(self.PATH)
        cursor = connect.cursor()
        data = [url, title, status, poster_path]
        cursor.execute(f'INSERT INTO {self.TABLE}(Url, Title, Status, PosterPath) VALUES(?,?,?,?)', data)
        connect.commit()
        cursor.close()
        connect.close()

    def update_anime_status(self, url, new_status):
        """Add entry of a content."""
        connect = sqlite3.connect(self.PATH)
        cursor = connect.cursor()
        cursor.execute(f"UPDATE {self.TABLE} SET Status = '{new_status}' WHERE Url = '{url}'")
        connect.commit()
        cursor.close()
        connect.close()

    def get_entry(self, url):
        """
        Returns the enties as an list of tuples if exists.
        Otherwise, return [].
        """
        connect = sqlite3.connect(self.PATH)
        cursor = connect.cursor()
        cursor.execute(f"SELECT * FROM {self.TABLE} WHERE Url = '{url}';")
        data = cursor.fetchall()
        cursor.close()
        connect.close()
        return data

    def add_anime_if(self, url, title, status, poster_path=""):
        """
        Return 0 if not exists.
        Return 1 if the same as the entry in the DB.
        Return -1 if the status is newer than the entry status.
        """
        data = self.get_entry(url)
        # If not exists
        if data == []:
            self.add_anime(url, title, status, poster_path)
            return 0
        # If the same
        if data[0][1:] == (url, title, status, poster_path):
            return 1
        # If the status is newer than the entry status.
        if data[0][3] != status:
            self.update_anime_status(url, status)
            return -1



##########################################################################
# Copyright (C) 2022-2023 Kostiantyn Klochko <kklochko@protonmail.com>   #
#                                                                        #
# This file is part of Anitube Simple Notification.                      #
#                                                                        #
# Anitube Simple Notification is free software: you can redistribute     #
# it and/or modify it under the terms of the GNU General Public          #
# License as published by the Free Software Foundation, either version   #
# 3 of the License, or (at your option) any later version.               #
#                                                                        #
# Anitube Simple Notification is distributed in the hope that it will    #
# be useful, but WITHOUT ANY WARRANTY; without even the implied          #
# warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See   #
# the GNU General Public License for more details.                       #
#                                                                        #
# You should have received a copy of the GNU General Public License      #
# along with Anitube Simple Notification. If not, see                    #
# <https://www.gnu.org/licenses/>.                                       #
##########################################################################

"""
This module has all for simplify work with the toml configuration file.
"""

from rich.console import Console
import tomli
from pathlib import PosixPath

class Config:
    @staticmethod
    def config_validation(config: dict) -> bool:
        """Return False if config is invalid. Empty values is valid."""
        validations = {
            "POSTERS": lambda v: isinstance(v, bool),
            "WAITING_PERIOD": lambda v: isinstance(v, int) and v >= 0,
            "URLS": lambda l: isinstance(l, list) and len(l) == len(list(filter(str,l))),
        }
        return all(validations.get(k, lambda x: False)(v) for (k, v) in config.items())

    @staticmethod
    def config_validation_detail(config: dict) -> dict:
        """Return a dict where key is a field and value is valid status."""
        validations = {
            "POSTERS": lambda v: isinstance(v, bool),
            "WAITING_PERIOD": lambda v: isinstance(v, int) and v >= 0,
            "URLS": lambda l: isinstance(l, list) and
            len(list(filter(lambda v: isinstance(v,str), l))) and
            len(list(filter(lambda v: v.startswith("https://anitube.in.ua/"), l)))
            #len(l) == len(list(filter(str,l))),
        }
        return {k:validations.get(k, lambda x: False)(v) for (k, v) in config.items()}

    @staticmethod
    def config_validation_error(config: dict):
        """Show error and remove invalid fields."""
        validation = Config.config_validation_detail(config)
        if not validation.pop("POSTERS", True):
            Console().print(f"[red][ERROR] Please, POSTER must be false or true.[/]")
            del config["POSTERS"]
        if not validation.pop("WAITING_PERIOD", True):
            Console().print(
                ''.join(["[red][ERROR] ",
                         "Please, WAITING_PERIOD must be an integer that greater",
                         "than zero or equals to.[/]"]))
            del config["WAITING_PERIOD"]
        if not validation.pop("URLS", True):
            Console().print(
                "".join(["[red][ERROR] Please, URLS must be a list of strings"
                         "which ones are page urls of anitube.in.ua.[/]"]))
            del config["URLS"]

    @staticmethod
    def get_config(config_path:PosixPath, console:Console) -> dict:
        """
        Read the configuration file and return dict as configuration.
        The configuration file must be in the application folder.
        """
        config = {}
        try:
            with open(config_path, "rb") as file:
                config = tomli.load(file)
        except FileNotFoundError:
            console.print(f"[red][ERROR] Please, create configuration file.[/]")
            console.print(f"[yellow][ERROR] The configuration file path: {config_path}.[/]")
        except tomli.TOMLDecodeError:
            console.print(f"[red][ERROR] Please, check configuration file for correctness.[/]")
        Config.config_validation_error(config)
        return config
